// ;-*-C++-*- *  Time-stamp: "2010-02-23 19:46:37 hmmr"
/*
 * Author: Andrei Zavada (a.zavada@sussex.ac.uk)
 *         building on original work by Thomas Nowotny (tnowotny@ucsd.edu)
 *
 * License: GPL
 *
 * Initial version: 2008-08-02
 *
 */



#ifndef LIBCN_STANDALONE_SYNAPSES_H
#define LIBCN_STANDALONE_SYNAPSES_H

#include <iostream>

#include "base-synapse.h"
#include "base-neuron.h"
#include "standalone-attr.h"
#include "mx-attr.h"


namespace CNRun {

class CModel;

class C_StandaloneSynapse
  : public C_BaseSynapse, public C_StandaloneAttributes {

    private:
	C_StandaloneSynapse();
    protected:
	C_StandaloneSynapse( TUnitType intype, C_BaseNeuron *insource, C_BaseNeuron *intarget,
			     double ing, CModel* inM, int s_mask = 0);

    public:
       ~C_StandaloneSynapse();

	double &var_value( size_t v)			{ return V[v]; }
	const double &get_var_value( size_t v) const	{ return V[v]; }
	double  S() const				{ return V[0]; }
	double &S( vector<double>&)			{ return V[0];  }

	void reset_vars()
		{
			memcpy( V.data(), __CNUDT[_type].stock_var_values,
				sizeof(double) * v_no());
			memcpy( V_next.data(), __CNUDT[_type].stock_var_values,
				sizeof(double) * v_no());
		}
};





class CSynapseMap
  : public C_StandaloneSynapse {

    private:
	CSynapseMap();

    public:
	CSynapseMap( C_BaseNeuron *insource, C_BaseNeuron *intarget,
		     double ing, CModel *inM, int s_mask = 0, TUnitType alt_type = YT_MAP)
	      : C_StandaloneSynapse( alt_type, insource, intarget, ing, inM, s_mask),
		_source_was_spiking (false)
		{
			if ( !inM )
				cerr << "A Map synapse is created unattached to a model: preadvance() will cause a segfault!\n";
		}

	void preadvance();  // defined inline in model.h

	enum {
		_tau_, _delta_, _Esyn_
	};
	double Isyn( const C_BaseNeuron &with_neuron, double g) const
		{
			return -g * S() * (with_neuron.E() - P[_Esyn_]);
		}
	double Isyn( vector<double>& unused, const C_BaseNeuron &with_neuron, double g) const
		{
			return Isyn( with_neuron, g);
		}

    protected:
	bool _source_was_spiking;
};





class CSynapseMxMap
  : public CSynapseMap, public C_MultiplexingAttributes {

    public:
	CSynapseMxMap( C_BaseNeuron *insource, C_BaseNeuron *intarget,
		       double ing, CModel *inM, int s_mask = 0)
	      : CSynapseMap( insource, intarget, ing, inM, s_mask, YT_MXMAP)
		{
			if ( !inM )
				fprintf( stderr, "A MxMap synapse is created unattached to a model: preadvance() will cause a segfault!\n");
		}

	enum {
		_tau_, _delta_, _Esyn_
	};
	void preadvance();  // defined inline in model.h

    private:
	friend class CModel;
	void update_queue();
};

}

#endif

// EOF
